#from distutils.core import setup
from setuptools import setup

setup(name='tweetfeels',
      version='0.1.2',
      description='Real-time sentiment analysis for twitter.',
      author='Thomas Chen',
      author_email='tkchen@gmail.com',
      url='https://github.com/uclatommy/tweetfeels',
      download_url='https://github.com/uclatommy/tweetfeels/tarball/0.1.2',
      packages=['tweetfeels']
     )
